@@ -1,42 +1,20 @@
-;; This file is part of Malaga, a system for Natural Language Analysis.
-;; Copyright (C) 1995-1999 Bjoern Beutel
-;;
-;; Bjoern Beutel
-;; Universitaet Erlangen-Nuernberg
-;; Abteilung fuer Computerlinguistik
-;; Bismarckstrasse 12
-;; D-91054 Erlangen
-;; e-mail: malaga@linguistik.uni-erlangen.de 
-;;
-;; This program is free software; you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2 of the License, or
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;; Copyright (C) 1995 Bjoern Beutel.
 
 (provide 'malaga)
 
-(defconst malaga-version "4.3" "Version of Malaga.")
+(defconst malaga-version "6.13" "Version of Malaga.")
 
-;; malaga mode (for editing malaga files) =====================================
+;; malaga mode (for editing malaga files). ====================================
 
-(setq auto-mode-alist ; files for which malaga mode will be invoked
+(setq auto-mode-alist ; Files for which malaga mode will be invoked.
       (append '(("\\.all$" . malaga-mode)
-                ("\\.mor$" . malaga-mode)
-                ("\\.syn$" . malaga-mode)
-		("\\.mal$" . malaga-mode)
-		("\\.lex$" . malaga-mode)
-		("\\.sym$" . malaga-mode)
                 ("\\.esym$" . malaga-mode)
-		("\\.pro$" . text-mode)
+		("\\.lex$" . malaga-mode)
+		("\\.mal$" . malaga-mode)
+                ("\\.mor$" . malaga-mode)
+		("\\.sym$" . malaga-mode)
+                ("\\.syn$" . malaga-mode)
+		("\\.pro$" . malaga-project-mode)
                 ) auto-mode-alist))
 
 ;;-----------------------------------------------------------------------------
@@ -47,7 +25,6 @@
 Key bindings:
   \\[malaga-electric-tab], \\[malaga-electric-semicolon] \
 and \\[malaga-electric-terminate-line] indent code.
-  \\[malaga-complete-keyword] completes a malaga keyword.
 
   \\[malaga-goto-previous-rule] jumps to previous malaga rule.
   \\[malaga-goto-next-rule] jumps to next malaga rule.
@@ -67,8 +44,7 @@ Variables controlling malaga-mode for all buffers:
   malaga-indent-level
   malaga-brace-level
   malaga-tab-always-indent
-  malaga-change-to-project-directory
-  malaga-upcase-completion"
+  malaga-change-to-project-directory"
   
   (interactive)
   (kill-all-local-variables)
@@ -96,46 +72,46 @@ Variables controlling malaga-mode for all buffers:
 
 (defun malaga-init-menu ()
   "Initialize menu for malaga mode."
+  (let ((malaga-items
+	'("Malaga"
+	  ["Previous rule" malaga-goto-previous-rule t]
+	  ["Next rule" malaga-goto-next-rule t]
+	  ["Jump to rule..." malaga-goto-rule t]
+	  "-----"
+	  ["Comment region" comment-region (mark)]
+	  ["Uncomment region" 
+	   (comment-region (region-beginning) (region-end) -1) 
+	   (mark)]
+	  "-----"
+	  ["UPCASE keywords" malaga-upcase-keywords t]
+	  ["downcase keywords" malaga-downcase-keywords t]
+	  "-----"
+	  ["Call malmake..." malmake t]
+	  ["Call mallex..." mallex t]
+	  ["Call malaga..." malaga t]
+       ["Change to project directory" 
+       (setq malaga-change-to-project-directory
+	     (not malaga-change-to-project-directory))
+       :style toggle :selected malaga-change-to-project-directory]
+       )))
 
-  (if (not (string-match "XEmacs\\|Lucid" emacs-version))
-      ()
-    (set-buffer-menubar (copy-sequence current-menubar))
+    (cond ((string-match "XEmacs\\|Lucid" emacs-version)
+	   (set-buffer-menubar (copy-sequence current-menubar))
+	   (add-submenu nil malaga-items))
+	  (t
+	   (require 'easymenu)
+	   (easy-menu-define malaga-menu malaga-mode-map 
+	     "Malaga menu" malaga-items)))))
 
-    (add-submenu nil
-		 '("Malaga"
-		   ["Previous rule" malaga-goto-previous-rule t]
-		   ["Next rule" malaga-goto-next-rule t]
-		   ["Jump to rule..." malaga-goto-rule t]
-		   "-----"
-		   ["Comment region" comment-region (mark)]
-		   ["Uncomment region" 
-		    (comment-region (region-beginning) (region-end) -1) 
-		    (mark)]
-		   "-----"
-		   ["UPCASE keywords" malaga-upcase-keywords t]
-		   ["downcase keywords" malaga-downcase-keywords t]
-		   ["UPCASE completion" (setq malaga-upcase-completion
-					      (not malaga-upcase-completion)) 
-		    :style toggle :selected malaga-upcase-completion]
-		   "-----"
-		   ["Call malmake..." malmake t]
-		   ["Call mallex..." mallex t]
-		   ["Call malaga..." malaga t]
-		   ["Change to project directory" 
-		    (setq malaga-change-to-project-directory
-			  (not malaga-change-to-project-directory))
-		    :style toggle :selected malaga-change-to-project-directory]
-		   ))))
-
-;; Data Structures for malaga mode ============================================
+;; Data Structures for malaga mode. ===========================================
 
 (defconst malaga-keywords-list
-  '("accept" "allo_rule" "and" "assert" "choose" "combi_rule" "define" "else"
-    "elseif" "end" "end_rule" "error" "fail" "foreach" "greater" 
+  '("accept" "allo_rule" "and" "assert" "break" "choose" "combi_rule" "default"
+    "define" "else" "elseif" "end" "end_rule" "error" "foreach" "greater" 
     "greater_equal" "if" "in" "include" "initial" "input_filter" "less" 
     "less_equal" "matches" "not" "or" "output_filter" "parallel" "pruning_rule"
-    "repeat" "require" "result" "return" "robust_rule" "rules" "subrule" "then"
-    "while")
+    "repeat" "require" "result" "return" "robust_rule" "rules" "stop" "subrule"
+    "then" "while")
   "The keywords used in malaga rule files (as a list).")
 
 (defvar malaga-keywords nil 
@@ -149,11 +125,12 @@ Variables controlling malaga-mode for all buffers:
   (concat "\\<\\(allo_rule\\|combi_rule\\|end_rule\\|input_filter"
 	  "\\|output_filter\\|pruning_rule\\|robust_rule\\|subrule\\)\\>"))
 (defconst malaga-begin-block-re 
-  (concat "\\(" malaga-rule-re "[\t ]*" malaga-ident-re "[\t ]*(\\)"
+  (concat "\\(" malaga-rule-re "[\t ]*\\({\\|" malaga-ident-re "[\t ]*(\\)\\)"
 	  "\\|\\<\\(foreach\\|if\\|parallel\\|repeat\\)\\>[\t ]*[^\t ;]"))
 (defconst malaga-end-block-re "\\<end\\>")
 (defconst malaga-sub-block-re "\\<\\(and\\|else\\|elseif\\|while\\)\\>")
-(defconst malaga-noindent-re "\\<\\(and\\|else\\|elseif\\|end\\|while\\)\\>")
+(defconst malaga-noindent-re 
+  "\\<\\(and\\|else\\|elseif\\|end\\|while\\|then\\)\\>")
 (defconst malaga-autoindent-lines-re
   (concat "\\<\\(and\\|[A-Za-z_]+rule\\|end\\|else\\|elseif\\|foreach\\|if"
 	  "\\|of\\|parallel\\|while\\)\\>"))
@@ -195,8 +172,9 @@ Variables controlling malaga-mode for all buffers:
 (define-key malaga-mode-map "\t" 'malaga-electric-tab)
 (define-key malaga-mode-map ";" 'malaga-electric-semicolon)
 (define-key malaga-mode-map "\r" 'malaga-electric-terminate-line)
-(define-key malaga-mode-map "\177" 'backward-delete-char-untabify)
-(define-key malaga-mode-map "\M-\t" 'malaga-complete-keyword)
+(if (string-match "XEmacs\\|Lucid" emacs-version)
+    (define-key malaga-mode-map "\C-h" 'backward-delete-char-untabify)
+  (define-key malaga-mode-map "\177" 'backward-delete-char-untabify))
 (define-key malaga-mode-map "\C-c\C-d" 'malaga-delete-arrow)
 (define-key malaga-mode-map "\C-c\C-p" 'malmake)
 (define-key malaga-mode-map "\C-c\C-r" 'malaga)
@@ -206,7 +184,7 @@ Variables controlling malaga-mode for all buffers:
 (define-key malaga-mode-map "\M-p" 'malaga-goto-previous-rule)
 (define-key malaga-mode-map "\M-g" 'malaga-goto-rule)
 
-;; User configuration for malaga mode =========================================
+;; User configuration for malaga mode. ========================================
 
 (defconst malaga-indent-level 2
   "*Indentation of Malaga statements with respect to containing block.")
@@ -219,43 +197,40 @@ Variables controlling malaga-mode for all buffers:
    current line, regardless of where in the line point is when the TAB
    command is used.")
 
-(defconst malaga-upcase-completion nil
-  "*If set to non-nil, Malaga keyword completion is in uppercase.")
-
-;; Font-lock functions ========================================================
+;; Font-lock functions. =======================================================
 
 (defconst malaga-font-lock-keywords
-  (list (list (concat "\\(\$" malaga-ident-re "\\>\\)") 
+  (list (list (concat "\\([@\$]" malaga-ident-re "\\>\\)") 
 	      1 'font-lock-variable-name-face)
 	(list (concat "\\<\\(" (mapconcat 'identity malaga-keywords-list "\\|")
 		      "\\)\\>")
 	      1 'font-lock-keyword-face))
-  "Expressions to highlight in Malaga mode")
+  "Expressions to highlight in Malaga mode.")
 
 (defun malaga-font ()
   "Set font-lock variables for malaga mode."
   
-  (make-local-variable 'font-lock-keywords-case-fold-search) ; for FSF Emacs
+  (make-local-variable 'font-lock-keywords-case-fold-search) ; For GNU Emacs.
   (setq font-lock-keywords-case-fold-search t)
-  (put major-mode 'font-lock-keywords-case-fold-search t) ; for XEmacs
+  (put major-mode 'font-lock-keywords-case-fold-search t) ; For XEmacs.
 
-  (make-local-variable 'font-lock-keywords)
-  (setq font-lock-keywords malaga-font-lock-keywords))
+  (make-local-variable 'font-lock-defaults)
+  (setq font-lock-defaults '(malaga-font-lock-keywords nil t)))
 
-;; Electric functions =========================================================
+;; Electric keys. =============================================================
 
 (defun malaga-electric-terminate-line ()
   "Terminate line and indent next line."
   (interactive)
 
-  ;; Check if current line should be indented
+  ;; Check if current line should be indented.
   (save-excursion
     (beginning-of-line)
     (skip-chars-forward " \t")
     (if (looking-at malaga-autoindent-lines-re)
 	(malaga-indent-line)))
 
-  ;; Remove trailing whitespaces
+  ;; Remove trailing whitespaces.
   (delete-horizontal-space)
 
   (newline)
@@ -285,7 +260,7 @@ Variables controlling malaga-mode for all buffers:
     (if (> old-point (point))
 	(goto-char old-point))))
 
-;; Other malaga mode functions ================================================
+;; Other malaga mode functions. ===============================================
 
 (defun malaga-change-keywords (change-word)
   "Apply function CHANGE-WORD to all keywords in current buffer."
@@ -317,7 +292,7 @@ Variables controlling malaga-mode for all buffers:
   "Go to beginning of previous rule in current buffer."
   (interactive)
   (if (not (search-backward-regexp (concat "^" malaga-rule-re) nil t))
-      (message "No previous rule")))
+      (message "No previous rule.")))
 
 (defun malaga-goto-next-rule ()
   "Go to beginning of next rule in current buffer."
@@ -327,7 +302,7 @@ Variables controlling malaga-mode for all buffers:
     (if (search-forward-regexp (concat "^" malaga-rule-re) nil t)
 	(beginning-of-line)
       (goto-char old-point)
-      (message "No next rule"))))
+      (message "No next rule."))))
 
 (defun malaga-goto-rule (rule-name)
   "Go to beginning of RULE-NAME in current buffer."
@@ -335,17 +310,17 @@ Variables controlling malaga-mode for all buffers:
   (let ((old-point (point)))
     (goto-char (point-min))
     (if (search-forward-regexp 
-	 (concat "^" malaga-rule-re "\\s-+" rule-name) nil t)
+	 (concat "^" malaga-rule-re "[\t {]+" rule-name) nil t)
 	(beginning-of-line)
       (goto-char old-point)
-      (message "No rule \"%s\"" rule-name))))
+      (message "No rule \"%s\"." rule-name))))
 
 (defun malaga-delete-arrow ()
   "Delete the source file arrow for the current buffer."
   (interactive)
   (setq overlay-arrow-string nil))
 
-;; Indentation ================================================================
+;; Indentation. ===============================================================
 
 (defconst malaga-indent-alist
   '((block . (+ ind malaga-indent-level))
@@ -386,7 +361,7 @@ Variables controlling malaga-mode for all buffers:
 	   (indent malaga-brace-level)
 	   (complete nil)
 	   (type (catch 'nesting
-		   ;; Check if inside a string or parenthesis
+		   ;; Check if inside a string or parenthesis.
 		   (cond ((nth 3 state) (throw 'nesting 'string))
 			 ((nth 4 state) (throw 'nesting 'comment))
 			 ((> (car state) 0)
@@ -395,92 +370,77 @@ Variables controlling malaga-mode for all buffers:
 			  (setq par (+ indent (current-column)))
 			  (throw 'nesting 'contexp)))
 
-		   ;; Loop until correct indent is found
+		   ;; Loop until correct indent is found.
 		   (while t
 		     (backward-sexp 1)
-		     (cond (; Nest block outwards
+		     (cond (; Nest block outwards.
 			    (looking-at malaga-begin-block-re)
 			    (if (= nest 0)
 				(throw 'nesting 'block)
 			      (setq nest (1- nest))))
 
-			   (; Nest block inwards
+			   (; Nest block inwards.
 			    (looking-at malaga-end-block-re)
 			    (setq complete t
 				  nest (1+ nest)))
 
-			   (; If, else or foreach statement
+			   (; If, else or foreach statement.
 			    (and (not complete)
 				 (looking-at malaga-sub-block-re))
 			    (throw 'nesting 'block))
 
-			   (; No known statements
+			   (; No known statements.
 			    (bobp)
 			    (throw 'nesting 'unknown))
 
-			   (; Found complete statement
+			   (; Found complete statement.
 			    (save-excursion (forward-sexp 1)
 					    (= (following-char) ?\;))
 			    (setq complete t))
 
-			   (; No known statements
+			   (; No known statements.
 			    (= (point) (point-min))
 			    (throw 'nesting 'unknown)))))))
 
       ;; Return type of block and indent level.
-      (if (> par 0) ; Unclosed Parenthesis 
+      (if (> par 0) ; Unclosed Parenthesis.
 	  (list 'contexp par)
 	(list type (malaga-indent-level))))))
 
 ;;-----------------------------------------------------------------------------
 
 (defun malaga-indent-level ()
-  "Return the indent-level the current statement has."
+  "Return the indent-level of the current statement."
   (save-excursion
     (beginning-of-line)
     (skip-chars-forward " \t")
     (current-column)))
 
-;;-----------------------------------------------------------------------------
+;; Malaga project mode. =======================================================
+
+(defvar malaga-project-mode-map ()
+  "Keymap used in Malaga project mode.")
+(setq malaga-project-mode-map (make-sparse-keymap))
+(define-key malaga-project-mode-map "\C-c\C-p" 'malmake)
+(define-key malaga-project-mode-map "\C-c\C-r" 'malaga)
+(define-key malaga-project-mode-map "\C-c\C-l" 'mallex)
+
+(defun malaga-project-mode ()
+  "Major mode for editing Malaga project files.
+  \\<malaga-project-mode-map>
+Key bindings:
+  \\[malmake] creates malmake process.
+  \\[malaga] creates malaga process.
+  \\[mallex] creates mallex process."
 
-(defun malaga-complete-keyword ()
-  "Perform completion on Malaga keyword preceding point."
   (interactive)
-  (let* ((end (point))
-	 (buffer-syntax (syntax-table))
-	 (beg (unwind-protect
-		  (save-excursion
-		    (set-syntax-table malaga-mode-syntax-table)
-		    (backward-sexp 1)
-		    (while (= (char-syntax (following-char)) ?\')
-		      (forward-char 1))
-		    (point))
-		(set-syntax-table buffer-syntax)))
-	 (pattern (buffer-substring beg end))
-	 
-	 (completion (try-completion pattern malaga-keywords)))
+  (kill-all-local-variables)
+  (setq major-mode 'malaga-project-mode)
+  (setq mode-name "Malaga-project")
+  (use-local-map malaga-project-mode-map)
+  (run-hooks 'malaga-project-mode-hook))
 
-    (cond ((eq completion t))
-	  
-	  ((null completion)
-	   (message "Can't find completion for \"%s\"" pattern)
-	   (ding))
-	  
-	  ((not (string= pattern completion))
-	   (delete-region beg end)
-	   
-	   (if (not (eq malaga-upcase-completion nil))
-	       (insert (upcase completion))
-	     (insert completion)))
-	  
-	  (t
-	   (message "Making completion list...")
-	   (let ((list (all-completions pattern malaga-keywords)))
-	     (with-output-to-temp-buffer "*Completions*"
-	       (display-completion-list list)))
-	   (message "Making completion list...%s" "done")))))
-
-;; Malaga process mode ========================================================
+;; Malaga process mode. =======================================================
 
 (require 'comint)
 
@@ -488,7 +448,7 @@ Variables controlling malaga-mode for all buffers:
   "Where malaga-display-line should put the debugging arrow.")
 
 (defvar malaga-process-mode-map ()
-  "Keymap used in Malaga process.")
+  "Keymap used in Malaga process mode.")
 (setq malaga-process-mode-map (copy-keymap comint-mode-map))
 (define-key malaga-process-mode-map "\t" 'comint-dynamic-complete-filename)
 (define-key malaga-process-mode-map "\C-c\C-d" 'malaga-delete-arrow)
@@ -532,15 +492,15 @@ Variables controlling malaga-process-mode for all buffers:
    The sentinel receives two arguments: the process malaga
    and a string describing the type of event, normally finished."
   (cond ((null (buffer-name (process-buffer proc)))
-	 ;; buffer killed
+	 ;; Buffer killed.
 	 ;; Stop displaying an arrow in a source file.
 	 (setq overlay-arrow-string nil)
 	 (set-process-buffer proc nil))
 	((memq (process-status proc) '(signal exit))
 	 (setq overlay-arrow-string nil)
 	 (let* ((obuf (current-buffer)))
-	   ;; save-excursion isn't the right thing 
-	   ;; if process-buffer is current-buffer
+	   ;; Save-excursion isn't the right thing 
+	   ;; if process-buffer is current-buffer.
 	   (unwind-protect
 	       (progn
 		 (set-buffer (process-buffer proc))
@@ -548,7 +508,7 @@ Variables controlling malaga-process-mode for all buffers:
 		 ;; Fix the mode line.
 		 (setq mode-line-process
 		       (concat ": " (symbol-name (process-status proc))))
-		 ;; Force mode line redisplay soon
+		 ;; Force mode line redisplay soon.
 		 (set-buffer-modified-p (buffer-modified-p))
 		 (if (eobp)
 		     (insert ?\n mode-name " " msg)
@@ -604,29 +564,21 @@ Variables controlling malaga-process-mode for all buffers:
 
 (defun malaga-marker-filter (string)
   "Detect file/line markers in STRING."
-  (let* ((line "") 
-	 (new-string ""))
-    
-    ;; divide string into lines and test each line separately
-    (while (or (string-match "\\`\\(.*\n\\)" string)
-	       (string-match "\\`\\(.+\\)\\'" string))
-      (setq line (substring string (match-beginning 1) (match-end 1)))
-      (setq string (substring string (match-end 1)))
-      
-      (cond ((string-match "SHOW \"\\([^\"]+\\)\":\\([0-9]+\\):\\([0-9]+\\)\n" 
-			   line)
-	     (setq malaga-last-frame 
-		   (list (substring line (match-beginning 1) (match-end 1))
-			 (string-to-int (substring line 
-						   (match-beginning 2) 
-						   (match-end 2)))
-			 (string-to-int (substring line 
-						   (match-beginning 3) 
-						   (match-end 3))))))
-	    (t
-	     (setq new-string (concat new-string line)))))
-    
-    new-string))
+  (cond ((string-match 
+	  "\\(SHOW \"\\([^\"]+\\)\":\\([0-9]+\\):\\([0-9]+\\)\n\\)"
+	  string)
+
+	 (setq malaga-last-frame 
+	       (list (substring string (match-beginning 2) (match-end 2))
+		     (string-to-int (substring string
+					       (match-beginning 3) 
+					       (match-end 3)))
+		     (string-to-int (substring string 
+					       (match-beginning 4) 
+					       (match-end 4)))))
+	 (setq string (concat (substring string 0 (match-beginning 1))
+			      (substring string (match-end 1))))))
+  string)
 
 ;;-----------------------------------------------------------------------------
 
@@ -639,7 +591,7 @@ Variables controlling malaga-process-mode for all buffers:
       
     ;; Print the process output, checking for source file markers.
     (comint-output-filter proc (malaga-marker-filter string))
-    
+
     (set-buffer old-buffer)
 
     ;; Check for a filename-and-line number.
@@ -656,9 +608,7 @@ Variables controlling malaga-process-mode for all buffers:
   "Call malaga, mallex or malmake and create the appropriate buffer."
   (let ((buffer-name (concat "*" process-name "*"))
 	(process-environment process-environment)
-	(program-name (if malaga-hangul
-			  (concat process-name "-hangul")
-			process-name))
+	(program-name process-name)
 	process-buffer)
 
     (cond ((or (not (get-process process-name))
@@ -677,7 +627,7 @@ Variables controlling malaga-process-mode for all buffers:
 	   (if malaga-change-to-project-directory
 	       (setq default-directory (file-name-directory project-file)))
 
-	(insert "\nCurrent directory is " default-directory "\n\n")
+	(insert "\nCurrent directory: " default-directory "\n\n")
 	(setenv "MALAGA_MODE" "t")
 	(if option
 	    (make-comint process-name program-name nil project-file option)
@@ -692,7 +642,7 @@ Variables controlling malaga-process-mode for all buffers:
 	(set-process-sentinel (get-buffer-process process-buffer)
 			      'malaga-sentinel)))))
 
-;; Creating malaga-processes ==================================================
+;; Creating malaga-processes. =================================================
 
 (defvar malaga-project-file "" "The default malaga project file.")
 
@@ -700,9 +650,6 @@ Variables controlling malaga-process-mode for all buffers:
 "*If non-nil, the buffer directory is set to the project directory
 when malaga, mallex or malmake is started.")
 
-(defconst malaga-hangul nil
-"*If non-nil, the hangul versions of the Malaga programs are started.")
-
 (defun malaga-read-project-file-name ()
   "Read the name of the malaga project file."
   (setq malaga-project-file 
@@ -718,7 +665,7 @@ when malaga, mallex or malmake is started.")
 (defun mallex (project-file)
   "Run mallex with PROJECT-FILE in buffer *mallex*."
   (interactive (list (malaga-read-project-file-name)))
-  (malaga-init "mallex" project-file "-interactive"))
+  (malaga-init "mallex" project-file))
 
 (defun malmake (project-file)
   "Run malmake with PROJECT-FILE in buffer *malmake*."
@@ -726,4 +673,4 @@ when malaga, mallex or malmake is started.")
   (save-some-buffers)
   (malaga-init "malmake" project-file))
 
-;; end of file ================================================================
+;; End of file. ===============================================================
